@@ -745,8 +745,12 @@ x509_ca_init(void)
 		x509_ca_paths = g_list_append(NULL, g_build_filename(DATADIR,
 						   "ca-certs", NULL));
 #else
+# ifdef SSL_CERTIFICATES_DIR
+		x509_ca_paths = g_list_append(NULL, g_strdup(SSL_CERTIFICATES_DIR));
+# else
 		x509_ca_paths = g_list_append(NULL, g_build_filename(DATADIR,
 						   "purple", "ca-certs", NULL));
+# endif
 #endif
 	}
 
@@ -1481,7 +1485,7 @@ x509_tls_cached_start_verify(PurpleCertificateVerificationRequest *vrq)
 
 	if (!tls_peers) {
 		purple_debug_error("certificate/x509/tls_cached",
-				   "Couldn't find local peers cache %s\nPrompting the user\n",
+				   "Couldn't find local peers cache %s\n",
 				   tls_peers_name);
 
 
