@@ -46,8 +46,12 @@
 
 #include <glib.h>
 
+/* If you're using this as the basis of a prpl that will be distributed
+ * separately from libpurple, remove the internal.h include below and replace
+ * it with code to include your own config.h or similar.  If you're going to
+ * provide for translation, you'll also need to setup the gettext macros. */
 #include "internal.h"
-#include "config.h"
+
 #include "account.h"
 #include "accountopt.h"
 #include "blist.h"
@@ -1125,7 +1129,8 @@ static PurplePluginProtocolInfo prpl_info =
   NULL,                                /* padding... */
   NULL,
   NULL,
-  NULL,
+	sizeof(PurplePluginProtocolInfo),    /* struct_size */
+  NULL
 };
 
 static void nullprpl_init(PurplePlugin *plugin)
