@@ -29,11 +29,11 @@
 
 #include "utils.h"
 #include "packet_parse.h"
+#include "buddy_list.h"
 #include "buddy_info.h"
 #include "char_conv.h"
-#include "crypt.h"
 #include "header_info.h"
-#include "keep_alive.h"
+#include "qq_base.h"
 #include "qq_network.h"
 
 #define QQ_PRIMARY_INFORMATION _("Primary Information")
@@ -85,6 +85,7 @@ static const gchar *genders[] = {
 };
 
 #define QQ_CONTACT_FIELDS                               37
+#define QQ_FACES	    100
 
 /* There is no user id stored in the reply packet for information query
  * we have to manually store the query, so that we know the query source */
@@ -208,8 +209,8 @@ static gboolean append_field_value(PurpleNotifyUserInfo *user_info, const gchar
 	return FALSE;
 }
 
-	static PurpleNotifyUserInfo *
-info_to_notify_user_info(const contact_info *info)
+static PurpleNotifyUserInfo *
+	info_to_notify_user_info(const contact_info *info)
 {
 	PurpleNotifyUserInfo *user_info = purple_notify_user_info_new();
 	const gchar *intro;
@@ -283,7 +284,7 @@ void qq_send_packet_get_info(PurpleConnection *gc, guint32 uid, gboolean show_wi
 
 	qd = (qq_data *) gc->proto_data;
 	g_snprintf(uid_str, sizeof(uid_str), "%d", uid);
-	qq_send_cmd(qd, QQ_CMD_GET_USER_INFO, (guint8 *) uid_str, strlen(uid_str));
+	qq_send_cmd(gc, QQ_CMD_GET_BUDDY_INFO, (guint8 *) uid_str, strlen(uid_str));
 
 	query = g_new0(qq_info_query, 1);
 	query->uid = uid;
@@ -292,6 +293,20 @@ void qq_send_packet_get_info(PurpleConnection *gc, guint32 uid, gboolean show_wi
 	qd->info_query = g_list_append(qd->info_query, query);
 }
 
+void qq_request_buddy_info(PurpleConnection *gc, guint32 uid,
+		gint update_class, guint32 ship32)
+{
+	qq_data *qd;
+	gchar raw_data[16] = {0};
+
+	g_return_if_fail(uid != 0);
+
+	qd = (qq_data *) gc->proto_data;
+	g_snprintf(raw_data, sizeof(raw_data), "%d", uid);
+	qq_send_cmd_mess(gc, QQ_CMD_GET_BUDDY_INFO, (guint8 *) raw_data, strlen(raw_data),
+			update_class, ship32);
+}
+
 /* set up the fields requesting personal information and send a get_info packet
  * for myself */
 void qq_prepare_modify_info(PurpleConnection *gc)
@@ -313,7 +328,6 @@ void qq_prepare_modify_info(PurpleConnection *gc)
 /* send packet to modify personal information */
 static void qq_send_packet_modify_info(PurpleConnection *gc, contact_info *info)
 {
-	qq_data *qd = (qq_data *) gc->proto_data;
 	gint bytes = 0;
 	guint8 raw_data[MAX_PACKET_SIZE - 128] = {0};
 	guint8 bar;
@@ -445,7 +459,7 @@ static void qq_send_packet_modify_info(PurpleConnection *gc, contact_info *info)
 
 	bytes += qq_put8(raw_data + bytes, bar);
 
-	qq_send_cmd(qd, QQ_CMD_UPDATE_INFO, raw_data, bytes);
+	qq_send_cmd(gc, QQ_CMD_UPDATE_INFO, raw_data, bytes);
 
 }
 
@@ -676,9 +690,9 @@ static void create_modify_info_dialogue(PurpleConnection *gc, const contact_info
 		mid->info->qq_show = g_strdup(info->qq_show);
 		mid->info->unknown6 = g_strdup(info->unknown6);
 
-		purple_request_fields(gc, _("Modify my information"),
-				_("Modify my information"), NULL, fields,
-				_("Update my information"), G_CALLBACK(modify_info_ok_cb),
+		purple_request_fields(gc, _("Modify information"),
+				_("Modify information"), NULL, fields,
+				_("Update information"), G_CALLBACK(modify_info_ok_cb),
 				_("Cancel"), G_CALLBACK(modify_info_cancel_cb),
 				purple_connection_get_account(gc), NULL, NULL,
 				mid);
@@ -686,26 +700,18 @@ static void create_modify_info_dialogue(PurpleConnection *gc, const contact_info
 }
 
 /* process the reply of modify_info packet */
-void qq_process_modify_info_reply(guint8 *buf, gint buf_len, PurpleConnection *gc)
+void qq_process_modify_info_reply(guint8 *data, gint data_len, PurpleConnection *gc)
 {
 	qq_data *qd;
-	gint len;
-	guint8 *data;
 
-	g_return_if_fail(buf != NULL && buf_len != 0);
+	g_return_if_fail(data != NULL && data_len != 0);
 
 	qd = (qq_data *) gc->proto_data;
-	len = buf_len;
-	data = g_newa(guint8, len);
 
-	if (qq_decrypt(buf, buf_len, qd->session_key, data, &len)) {
-		data[len] = '\0';
-		if (qd->uid == atoi((gchar *) data)) {	/* return should be my uid */
-			purple_debug(PURPLE_DEBUG_INFO, "QQ", "Update info ACK OK\n");
-			purple_notify_info(gc, NULL, _("Your information has been updated"), NULL);
-		}
-	} else {
-		purple_debug(PURPLE_DEBUG_ERROR, "QQ", "Error decrypt modify info reply\n");
+	data[data_len] = '\0';
+	if (qd->uid == atoi((gchar *) data)) {	/* return should be my uid */
+		purple_debug_info("QQ", "Update info ACK OK\n");
+		purple_notify_info(gc, _("QQ Buddy"), _("Successed:"), _("Change buddy information."));
 	}
 }
 
@@ -753,8 +759,8 @@ void qq_set_my_buddy_icon(PurpleConnection *gc, PurpleStoredImage *img)
 	const gchar *buddy_icon_dir = qq_buddy_icon_dir();
 	gint prefix_len = strlen(QQ_ICON_PREFIX);
 	gint suffix_len = strlen(QQ_ICON_SUFFIX);
-	gint dir_len = strlen(buddy_icon_dir);
-	gchar *errmsg = g_strdup_printf(_("Setting custom faces is not currently supported. Please choose an image from %s."), buddy_icon_dir);
+	gint dir_len = buddy_icon_dir ? strlen(buddy_icon_dir) : 0;
+	gchar *errmsg = g_strdup_printf(_("Setting custom faces is not currently supported. Please choose an image from %s."), buddy_icon_dir ? buddy_icon_dir : "(null)");
 	gboolean icon_global = purple_account_get_bool(gc->account, "use-global-buddyicon", TRUE);
 
 	if (!icon_path)
@@ -763,13 +769,13 @@ void qq_set_my_buddy_icon(PurpleConnection *gc, PurpleStoredImage *img)
 	icon_len = strlen(icon_path) - dir_len - 1 - prefix_len - suffix_len;
 
 	/* make sure we're using an appropriate icon */
-	if (!(g_ascii_strncasecmp(icon_path, buddy_icon_dir, dir_len) == 0
+	if (buddy_icon_dir && !(g_ascii_strncasecmp(icon_path, buddy_icon_dir, dir_len) == 0
 				&& icon_path[dir_len] == G_DIR_SEPARATOR
 				&& g_ascii_strncasecmp(icon_path + dir_len + 1, QQ_ICON_PREFIX, prefix_len) == 0
 				&& g_ascii_strncasecmp(icon_path + dir_len + 1 + prefix_len + icon_len, QQ_ICON_SUFFIX, suffix_len) == 0
 				&& icon_len <= 3)) {
 		if (icon_global)
-			purple_debug(PURPLE_DEBUG_ERROR, "QQ", "%s\n", errmsg);
+			purple_debug_error("QQ", "%s\n", errmsg);
 		else
 			purple_notify_error(gc, _("Invalid QQ Face"), errmsg, NULL);
 		g_free(errmsg);
@@ -782,7 +788,7 @@ void qq_set_my_buddy_icon(PurpleConnection *gc, PurpleStoredImage *img)
 	/* ensure face number in proper range */
 	if (icon_num > QQ_FACES) {
 		if (icon_global)
-			purple_debug(PURPLE_DEBUG_ERROR, "QQ", "%s\n", errmsg);
+			purple_debug_error("QQ", "%s\n", errmsg);
 		else
 			purple_notify_error(gc, _("Invalid QQ Face"), errmsg, NULL);
 		g_free(errmsg);
@@ -805,8 +811,8 @@ static void _qq_update_buddy_icon(PurpleAccount *account, const gchar *name, gin
 	if ((buddy = purple_find_buddy(account, name)))
 		old_icon_num = purple_buddy_icons_get_checksum_for_user(buddy);
 
-	if (old_icon_num == NULL ||
-			strcmp(icon_num_str, old_icon_num))
+	if ((old_icon_num == NULL ||
+			strcmp(icon_num_str, old_icon_num)) && (qq_buddy_icon_dir() != NULL))
 	{
 		gchar *icon_path;
 
@@ -826,7 +832,8 @@ static void qq_refresh_buddy_and_myself(contact_info *info, PurpleConnection *gc
 	PurpleBuddy *b;
 	qq_data *qd;
 	qq_buddy *q_bud;
-	gchar *alias_utf8, *purple_name;
+	gchar *alias_utf8;
+	gchar *purple_name;
 	PurpleAccount *account = purple_connection_get_account(gc);
 
 	qd = (qq_data *) gc->proto_data;
@@ -855,10 +862,8 @@ static void qq_refresh_buddy_and_myself(contact_info *info, PurpleConnection *gc
 }
 
 /* process reply to get_info packet */
-void qq_process_get_info_reply(guint8 *buf, gint buf_len, PurpleConnection *gc)
+void qq_process_get_buddy_info(guint8 *data, gint data_len, PurpleConnection *gc)
 {
-	gint len;
-	guint8 *data;
 	gchar **segments;
 	qq_info_query *query;
 	qq_data *qd;
@@ -866,72 +871,68 @@ void qq_process_get_info_reply(guint8 *buf, gint buf_len, PurpleConnection *gc)
 	GList *list, *query_list;
 	PurpleNotifyUserInfo *user_info;
 
-	g_return_if_fail(buf != NULL && buf_len != 0);
+	g_return_if_fail(data != NULL && data_len != 0);
 
 	qd = (qq_data *) gc->proto_data;
 	list = query_list = NULL;
-	len = buf_len;
-	data = g_newa(guint8, len);
 	info = NULL;
 
-	if (qq_decrypt(buf, buf_len, qd->session_key, data, &len)) {
-		if (NULL == (segments = split_data(data, len, "\x1e", QQ_CONTACT_FIELDS)))
-			return;
+	if (NULL == (segments = split_data(data, data_len, "\x1e", QQ_CONTACT_FIELDS)))
+		return;
 
-		info = (contact_info *) segments;
-		if (qd->modifying_face && strtol(info->face, NULL, 10) != qd->my_icon) {
-			gchar *icon = g_strdup_printf("%d", qd->my_icon);
-			qd->modifying_face = FALSE;
-			g_free(info->face);
-			info->face = icon;
-			qq_send_packet_modify_info(gc, (contact_info *)segments);
-		}
-
-		qq_refresh_buddy_and_myself(info, gc);
-
-		query_list = qd->info_query;
-		/* ensure we're processing the right query */
-		while (query_list) {
-			query = (qq_info_query *) query_list->data;
-			if (query->uid == atoi(info->uid)) {
-				if (query->show_window) {
-					user_info = info_to_notify_user_info(info);
-					purple_notify_userinfo(gc, info->uid, user_info, NULL, NULL);
-					purple_notify_user_info_destroy(user_info);
-				} else if (query->modify_info) {
-					create_modify_info_dialogue(gc, info);
-				}
-				qd->info_query = g_list_remove(qd->info_query, qd->info_query->data);
-				g_free(query);
-				break;
-			}
-			query_list = query_list->next;
-		}
-
-		g_strfreev(segments);
-	} else {
-		purple_debug(PURPLE_DEBUG_ERROR, "QQ", "Error decrypt get info reply\n");
+	info = (contact_info *) segments;
+	if (qd->modifying_face && strtol(info->face, NULL, 10) != qd->my_icon) {
+		gchar *icon = g_strdup_printf("%d", qd->my_icon);
+		qd->modifying_face = FALSE;
+		g_free(info->face);
+		info->face = icon;
+		qq_send_packet_modify_info(gc, (contact_info *)segments);
 	}
+
+	qq_refresh_buddy_and_myself(info, gc);
+
+	query_list = qd->info_query;
+	/* ensure we're processing the right query */
+	while (query_list) {
+		query = (qq_info_query *) query_list->data;
+		if (query->uid == atoi(info->uid)) {
+			if (query->show_window) {
+				user_info = info_to_notify_user_info(info);
+				purple_notify_userinfo(gc, info->uid, user_info, NULL, NULL);
+				purple_notify_user_info_destroy(user_info);
+			} else if (query->modify_info) {
+				create_modify_info_dialogue(gc, info);
+			}
+			qd->info_query = g_list_remove(qd->info_query, qd->info_query->data);
+			g_free(query);
+			break;
+		}
+		query_list = query_list->next;
+	}
+
+	g_strfreev(segments);
 }
 
 void qq_info_query_free(qq_data *qd)
 {
-	gint i;
+	gint count;
 	qq_info_query *p;
 
 	g_return_if_fail(qd != NULL);
 
-	i = 0;
+	count = 0;
 	while (qd->info_query != NULL) {
 		p = (qq_info_query *) (qd->info_query->data);
 		qd->info_query = g_list_remove(qd->info_query, p);
 		g_free(p);
-		i++;
+		count++;
+	}
+	if (count > 0) {
+		purple_debug_info("QQ", "%d info queries are freed!\n", count);
 	}
-	purple_debug(PURPLE_DEBUG_INFO, "QQ", "%d info queries are freed!\n", i);
 }
 
-void qq_send_packet_get_level(PurpleConnection *gc, guint32 uid)
+void qq_request_get_level(PurpleConnection *gc, guint32 uid)
 {
 	qq_data *qd = (qq_data *) gc->proto_data;
 	guint8 buf[16] = {0};
@@ -941,10 +942,10 @@ void qq_send_packet_get_level(PurpleConnection *gc, guint32 uid)
 	bytes += qq_put32(buf + bytes, uid);
 
 	qd = (qq_data *) gc->proto_data;
-	qq_send_cmd(qd, QQ_CMD_GET_LEVEL, buf, bytes);
+	qq_send_cmd(gc, QQ_CMD_GET_LEVEL, buf, bytes);
 }
 
-void qq_send_packet_get_buddies_levels(PurpleConnection *gc)
+void qq_request_get_buddies_level(PurpleConnection *gc, gint update_class)
 {
 	guint8 *buf;
 	guint16 size;
@@ -953,47 +954,42 @@ void qq_send_packet_get_buddies_levels(PurpleConnection *gc)
 	GList *node = qd->buddies;
 	gint bytes = 0;
 
-	if (qd->buddies) {
-		/* server only sends back levels for online buddies, no point
-		 * in asking for anyone else */
-		size = 4 * g_list_length(qd->buddies) + 1;
-		buf = g_new0(guint8, size);
-		bytes += 1;
-
-		while (NULL != node) {
-			q_bud = (qq_buddy *) node->data;
-			if (NULL != q_bud) {
-				bytes += qq_put32(buf + bytes, q_bud->uid);
-			}
-			node = node->next;
-		}
-		qq_send_cmd(qd, QQ_CMD_GET_LEVEL, buf, size);
-		g_free(buf);
+	if ( qd->buddies == NULL) {
+		return;
 	}
+	/* server only reply levels for online buddies */
+	size = 4 * g_list_length(qd->buddies) + 1 + 4;
+	buf = g_newa(guint8, size);
+	bytes += qq_put8(buf + bytes, 0x00);
+
+	while (NULL != node) {
+		q_bud = (qq_buddy *) node->data;
+		if (NULL != q_bud) {
+			bytes += qq_put32(buf + bytes, q_bud->uid);
+		}
+		node = node->next;
+	}
+
+	/* my id should be the end if included */
+	bytes += qq_put32(buf + bytes, qd->uid);
+	qq_send_cmd_mess(gc, QQ_CMD_GET_LEVEL, buf, size, update_class, 0);
 }
 
-void qq_process_get_level_reply(guint8 *buf, gint buf_len, PurpleConnection *gc)
+void qq_process_get_level_reply(guint8 *decr_buf, gint decr_len, PurpleConnection *gc)
 {
 	guint32 uid, onlineTime;
 	guint16 level, timeRemainder;
 	gchar *purple_name;
 	PurpleBuddy *b;
 	qq_buddy *q_bud;
-	gint decr_len, i;
-	guint8 *decr_buf;
+	gint i;
 	PurpleAccount *account = purple_connection_get_account(gc);
 	qq_data *qd = (qq_data *) gc->proto_data;
 	gint bytes = 0;
 
-	decr_len = buf_len;
-	decr_buf = g_new0(guint8, buf_len);
-	if (!qq_decrypt(buf, buf_len, qd->session_key, decr_buf, &decr_len)) {
-		purple_debug(PURPLE_DEBUG_ERROR, "QQ", "Couldn't decrypt get level packet\n");
-	}
-
-	decr_len--; 
+	decr_len--;
 	if (decr_len % 12 != 0) {
-		purple_debug(PURPLE_DEBUG_ERROR, "QQ", 
+		purple_debug_error("QQ",
 				"Get levels list of abnormal length. Truncating last %d bytes.\n", decr_len % 12);
 		decr_len -= (decr_len % 12);
 	}
@@ -1001,34 +997,42 @@ void qq_process_get_level_reply(guint8 *buf, gint buf_len, PurpleConnection *gc)
 	bytes += 1;
 	/* this byte seems random */
 	/*
-	   purple_debug(PURPLE_DEBUG_INFO, "QQ", "Byte one of get_level packet: %d\n", buf[0]);
+	   purple_debug_info("QQ", "Byte one of get_level packet: %d\n", buf[0]);
 	   */
 	for (i = 0; i < decr_len; i += 12) {
 		bytes += qq_get32(&uid, decr_buf + bytes);
 		bytes += qq_get32(&onlineTime, decr_buf + bytes);
 		bytes += qq_get16(&level, decr_buf + bytes);
 		bytes += qq_get16(&timeRemainder, decr_buf + bytes);
-		purple_debug(PURPLE_DEBUG_INFO, "QQ", 
-				"Level packet entry:\nuid: %d\nonlineTime: %d\nlevel: %d\ntimeRemainder: %d\n", 
+		purple_debug_info("QQ_LEVEL", "%d, tmOnline: %d, level: %d, tmRemainder: %d\n",
 				uid, onlineTime, level, timeRemainder);
-		purple_name = uid_to_purple_name(uid);
-		b = purple_find_buddy(account, purple_name);
-		q_bud = (b == NULL) ? NULL : (qq_buddy *) b->proto_data;
-
-		if (q_bud != NULL || uid == qd->uid) {
-			if (q_bud) {
-				q_bud->onlineTime = onlineTime;
-				q_bud->level = level;
-				q_bud->timeRemainder = timeRemainder;
-			}
-			if (uid == qd->uid) {
-				qd->my_level = level;
-			}
-		} else {
-			purple_debug(PURPLE_DEBUG_ERROR, "QQ", 
-					"Got an online buddy %d, but not in my buddy list\n", uid);
+		if (uid == qd->uid) {
+			qd->my_level = level;
+			purple_debug_warning("QQ", "Got my levels as %d\n", qd->my_level);
+			continue;
 		}
+
+		purple_name = uid_to_purple_name(uid);
+		if (purple_name == NULL) {
+			continue;
+		}
+
+		b = purple_find_buddy(account, purple_name);
 		g_free(purple_name);
+
+		q_bud = NULL;
+		if (b != NULL) {
+			q_bud = (qq_buddy *) b->proto_data;
+		}
+
+		if (q_bud == NULL) {
+			purple_debug_error("QQ", "Got levels of %d not in my buddy list\n", uid);
+			continue;
+		}
+
+		q_bud->onlineTime = onlineTime;
+		q_bud->level = level;
+		q_bud->timeRemainder = timeRemainder;
 	}
-	g_free(decr_buf);
 }
+
