@@ -29,12 +29,9 @@
 #include "connection.h"
 #include "group.h"
 
-void qq_send_cmd_group_get_online_members(PurpleConnection *gc, qq_group *group);
-void qq_send_cmd_group_all_get_online_members(PurpleConnection *gc);
-
-void qq_send_cmd_group_get_members_info(PurpleConnection *gc, qq_group *group);
+gint qq_request_room_get_buddies(PurpleConnection *gc, qq_group *group, gint update_class);
 
 void qq_process_room_cmd_get_info(guint8 *data, gint len, PurpleConnection *gc);
 void qq_process_room_cmd_get_onlines(guint8 *data, gint len, PurpleConnection *gc);
-void qq_process_room_cmd_get_members(guint8 *data, gint len, PurpleConnection *gc);
+void qq_process_room_cmd_get_buddies(guint8 *data, gint len, PurpleConnection *gc);
 #endif
