@@ -34,9 +34,11 @@
 #include "group_internal.h"
 #include "group_info.h"
 #include "group_join.h"
-#include "group_network.h"
 #include "group_opt.h"
+#include "header_info.h"
 #include "packet_parse.h"
+#include "qq_network.h"
+#include "qq_process.h"
 #include "utils.h"
 
 static int _compare_guint32(const void *a,
@@ -66,15 +68,13 @@ static void _qq_group_member_opt(PurpleConnection *gc, qq_group *group, gint ope
 	}
 	data_len = 6 + count * 4;
 	data = g_newa(guint8, data_len);
-	
+
 	bytes = 0;
-	bytes += qq_put8(data + bytes, QQ_GROUP_CMD_MEMBER_OPT);
-	bytes += qq_put32(data + bytes, group->internal_group_id);
 	bytes += qq_put8(data + bytes, operation);
 	for (i = 0; i < count; i++)
 		bytes += qq_put32(data + bytes, members[i]);
 
-	qq_send_group_cmd(gc, group, data, bytes);
+	qq_send_room_cmd(gc, QQ_ROOM_CMD_MEMBER_OPT, group->id, data, bytes);
 }
 
 static void _qq_group_do_nothing_with_struct(group_member_opt *g)
@@ -86,10 +86,10 @@ static void _qq_group_do_nothing_with_struct(group_member_opt *g)
 static void _qq_group_reject_application_real(group_member_opt *g, gchar *msg_utf8)
 {
 	qq_group *group;
-	g_return_if_fail(g != NULL && g->gc != NULL && g->internal_group_id > 0 && g->member > 0);
-	group = qq_group_find_by_id(g->gc, g->internal_group_id, QQ_INTERNAL_ID);
+	g_return_if_fail(g != NULL && g->gc != NULL && g->id > 0 && g->member > 0);
+	group = qq_room_search_id(g->gc, g->id);
 	g_return_if_fail(group != NULL);
-	qq_send_cmd_group_auth(g->gc, group, QQ_GROUP_AUTH_REQUEST_REJECT, g->member, msg_utf8);
+	qq_send_cmd_group_auth(g->gc, group, QQ_ROOM_AUTH_REQUEST_REJECT, g->member, msg_utf8);
 	g_free(g);
 }
 
@@ -112,11 +112,11 @@ void qq_group_reject_application_with_struct(group_member_opt *g)
 	g_return_if_fail(g != NULL && g->gc != NULL && g->member > 0);
 
 	msg1 = g_strdup_printf(_("You rejected %d's request"), g->member);
-	msg2 = g_strdup(_("Enter your reason:"));
+	msg2 = g_strdup(_("Message:"));
 
 	nombre = uid_to_purple_name(g->member);
 	purple_request_input(g->gc, /* title */ NULL, msg1, msg2,
-			   _("Sorry, you are not my type..."), /* multiline */ TRUE, /* masked */ FALSE,
+			   _("Sorry, you are not my style..."), /* multiline */ TRUE, /* masked */ FALSE,
 			   /* hint */ NULL,
 			   _("Send"), G_CALLBACK(_qq_group_reject_application_real),
 			   _("Cancel"), G_CALLBACK(_qq_group_do_nothing_with_struct),
@@ -131,10 +131,10 @@ void qq_group_reject_application_with_struct(group_member_opt *g)
 void qq_group_approve_application_with_struct(group_member_opt *g)
 {
 	qq_group *group;
-	g_return_if_fail(g != NULL && g->gc != NULL && g->internal_group_id > 0 && g->member > 0);
-	group = qq_group_find_by_id(g->gc, g->internal_group_id, QQ_INTERNAL_ID);
+	g_return_if_fail(g != NULL && g->gc != NULL && g->id > 0 && g->member > 0);
+	group = qq_room_search_id(g->gc, g->id);
 	g_return_if_fail(group != NULL);
-	qq_send_cmd_group_auth(g->gc, group, QQ_GROUP_AUTH_REQUEST_APPROVE, g->member, "");
+	qq_send_cmd_group_auth(g->gc, group, QQ_ROOM_AUTH_REQUEST_APPROVE, g->member, "");
 	qq_group_find_or_add_member(g->gc, group, g->member);
 	g_free(g);
 }
@@ -190,32 +190,32 @@ void qq_group_modify_members(PurpleConnection *gc, qq_group *group, guint32 *new
 		qq_group_find_or_add_member(gc, group, add_members[i]);
 
 	if (del > 0)
-		_qq_group_member_opt(gc, group, QQ_GROUP_MEMBER_DEL, del_members);
+		_qq_group_member_opt(gc, group, QQ_ROOM_MEMBER_DEL, del_members);
 	if (add > 0)
-		_qq_group_member_opt(gc, group, QQ_GROUP_MEMBER_ADD, add_members);
+		_qq_group_member_opt(gc, group, QQ_ROOM_MEMBER_ADD, add_members);
 }
 
 void qq_group_process_modify_members_reply(guint8 *data, gint len, PurpleConnection *gc)
 {
 	gint bytes;
-	guint32 internal_group_id;
+	guint32 id;
 	qq_group *group;
 	g_return_if_fail(data != NULL);
 
 	bytes = 0;
-	bytes += qq_get32(&internal_group_id, data + bytes);
-	g_return_if_fail(internal_group_id > 0);
+	bytes += qq_get32(&id, data + bytes);
+	g_return_if_fail(id > 0);
 
 	/* we should have its info locally */
-	group = qq_group_find_by_id(gc, internal_group_id, QQ_INTERNAL_ID);
+	group = qq_room_search_id(gc, id);
 	g_return_if_fail(group != NULL);
 
-	purple_debug(PURPLE_DEBUG_INFO, "QQ", "Succeed in modify members for Qun %d\n", group->external_group_id);
+	purple_debug_info("QQ", "Succeed in modify members for room %d\n", group->ext_id);
 
-	purple_notify_info(gc, _("QQ Qun Operation"), _("You have successfully modified Qun member"), NULL);
+	purple_notify_info(gc, _("QQ Qun Operation"), _("Successed:"), _("Change Qun member"));
 }
 
-void qq_group_modify_info(PurpleConnection *gc, qq_group *group)
+void qq_room_change_info(PurpleConnection *gc, qq_group *group)
 {
 	guint8 *data;
 	gint data_len;
@@ -224,20 +224,13 @@ void qq_group_modify_info(PurpleConnection *gc, qq_group *group)
 
 	g_return_if_fail(group != NULL);
 
-	group_name = group->group_name_utf8 == NULL ? "" : utf8_to_qq(group->group_name_utf8, QQ_CHARSET_DEFAULT);
-	group_desc = group->group_desc_utf8 == NULL ? "" : utf8_to_qq(group->group_desc_utf8, QQ_CHARSET_DEFAULT);
+	group_name = group->title_utf8 == NULL ? "" : utf8_to_qq(group->title_utf8, QQ_CHARSET_DEFAULT);
+	group_desc = group->desc_utf8 == NULL ? "" : utf8_to_qq(group->desc_utf8, QQ_CHARSET_DEFAULT);
 	notice = group->notice_utf8 == NULL ? "" : utf8_to_qq(group->notice_utf8, QQ_CHARSET_DEFAULT);
 
-	data_len = 13 + 1 + strlen(group_name)
-	    + 1 + strlen(group_desc)
-	    + 1 + strlen(notice);
-
+	data_len = 64 + strlen(group_name) + strlen(group_desc) + strlen(notice);
 	data = g_newa(guint8, data_len);
 	bytes = 0;
-	/* 000-000 */
-	bytes += qq_put8(data + bytes, QQ_GROUP_CMD_MODIFY_GROUP_INFO);
-	/* 001-004 */
-	bytes += qq_put32(data + bytes, group->internal_group_id);
 	/* 005-005 */
 	bytes += qq_put8(data + bytes, 0x01);
 	/* 006-006 */
@@ -245,7 +238,7 @@ void qq_group_modify_info(PurpleConnection *gc, qq_group *group)
 	/* 007-008 */
 	bytes += qq_put16(data + bytes, 0x0000);
 	/* 009-010 */
-	bytes += qq_put16(data + bytes, group->group_category);
+	bytes += qq_put16(data + bytes, group->category);
 
 	bytes += qq_put8(data + bytes, strlen(group_name));
 	bytes += qq_putdata(data + bytes, (guint8 *) group_name, strlen(group_name));
@@ -258,58 +251,56 @@ void qq_group_modify_info(PurpleConnection *gc, qq_group *group)
 	bytes += qq_put8(data + bytes, strlen(group_desc));
 	bytes += qq_putdata(data + bytes, (guint8 *) group_desc, strlen(group_desc));
 
-	if (bytes != data_len)	{
-		purple_debug(PURPLE_DEBUG_ERROR, "QQ",
-			   "Fail to create group_modify_info packet, expect %d bytes, wrote %d bytes\n",
+	if (bytes > data_len) {
+		purple_debug_error("QQ",
+			   "Overflow in qq_room_change_info, max %d bytes, now %d bytes\n",
 			   data_len, bytes);
 		return;
 	}
-
-	qq_send_group_cmd(gc, group, data, bytes);
+	qq_send_room_cmd(gc, QQ_ROOM_CMD_CHANGE_INFO, group->id, data, bytes);
 }
 
 void qq_group_process_modify_info_reply(guint8 *data, gint len, PurpleConnection *gc)
 {
 	gint bytes;
-	guint32 internal_group_id;
+	guint32 id;
 	qq_group *group;
 	g_return_if_fail(data != NULL);
 
 	bytes = 0;
-	bytes += qq_get32(&internal_group_id, data + bytes);
-	g_return_if_fail(internal_group_id > 0);
+	bytes += qq_get32(&id, data + bytes);
+	g_return_if_fail(id > 0);
 
 	/* we should have its info locally */
-	group = qq_group_find_by_id(gc, internal_group_id, QQ_INTERNAL_ID);
+	group = qq_room_search_id(gc, id);
 	g_return_if_fail(group != NULL);
 
-	purple_debug(PURPLE_DEBUG_INFO, "QQ", "Succeed in modify info for Qun %d\n", group->external_group_id);
+	purple_debug_info("QQ", "Succeed in modify info for Qun %d\n", group->ext_id);
 	qq_group_refresh(gc, group);
 
-	purple_notify_info(gc, _("QQ Qun Operation"), _("You have successfully modified Qun information"), NULL);
+	purple_notify_info(gc, _("QQ Qun Operation"), _("Successed:"), _("Change Qun information"));
 }
 
 /* we create a very simple group first, and then let the user to modify */
-void qq_group_create_with_name(PurpleConnection *gc, const gchar *name)
+void qq_room_create_new(PurpleConnection *gc, const gchar *name)
 {
-	gint data_len;
 	guint8 *data;
+	gint data_len;
 	gint bytes;
 	qq_data *qd;
 	g_return_if_fail(name != NULL);
 
 	qd = (qq_data *) gc->proto_data;
-	data_len = 7 + 1 + strlen(name) + 2 + 1 + 1 + 4;
+
+	data_len = 64 + strlen(name);
 	data = g_newa(guint8, data_len);
 
 	bytes = 0;
 	/* we create the simpleset group, only group name is given */
-	/* 000 */
-	bytes += qq_put8(data + bytes, QQ_GROUP_CMD_CREATE_GROUP);
 	/* 001 */
-	bytes += qq_put8(data + bytes, QQ_GROUP_TYPE_PERMANENT);
+	bytes += qq_put8(data + bytes, QQ_ROOM_TYPE_PERMANENT);
 	/* 002 */
-	bytes += qq_put8(data + bytes, QQ_GROUP_AUTH_TYPE_NEED_AUTH);
+	bytes += qq_put8(data + bytes, QQ_ROOM_AUTH_TYPE_NEED_AUTH);
 	/* 003-004 */
 	bytes += qq_put16(data + bytes, 0x0000);
 	/* 005-006 */
@@ -322,14 +313,13 @@ void qq_group_create_with_name(PurpleConnection *gc, const gchar *name)
 	bytes += qq_put8(data + bytes, 0x00);	/* no group desc */
 	bytes += qq_put32(data + bytes, qd->uid);	/* I am member of coz */
 
-	if (bytes != data_len) {
-		purple_debug(PURPLE_DEBUG_ERROR, "QQ",
-			   "Fail create create_group packet, expect %d bytes, written %d bytes\n",
+	if (bytes > data_len) {
+		purple_debug_error("QQ",
+			   "Overflow in qq_room_create, max %d bytes, now %d bytes\n",
 			   data_len, bytes);
 		return;
 	}
-
-	qq_send_group_cmd(gc, NULL, data, bytes);
+	qq_send_room_cmd_noid(gc, QQ_ROOM_CMD_CREATE, data, bytes);
 }
 
 static void qq_group_setup_with_gc_and_uid(gc_and_uid *g)
@@ -337,7 +327,7 @@ static void qq_group_setup_with_gc_and_uid(gc_and_uid *g)
 	qq_group *group;
 	g_return_if_fail(g != NULL && g->gc != NULL && g->uid > 0);
 
-	group = qq_group_find_by_id(g->gc, g->uid, QQ_INTERNAL_ID);
+	group = qq_room_search_id(g->gc, g->uid);
 	g_return_if_fail(group != NULL);
 
 	/* TODO insert UI code here */
@@ -348,7 +338,7 @@ static void qq_group_setup_with_gc_and_uid(gc_and_uid *g)
 void qq_group_process_create_group_reply(guint8 *data, gint len, PurpleConnection *gc)
 {
 	gint bytes;
-	guint32 internal_group_id, external_group_id;
+	guint32 id, ext_id;
 	qq_group *group;
 	gc_and_uid *g;
 	qq_data *qd;
@@ -358,28 +348,28 @@ void qq_group_process_create_group_reply(guint8 *data, gint len, PurpleConnectio
 	qd = (qq_data *) gc->proto_data;
 
 	bytes = 0;
-	bytes += qq_get32(&internal_group_id, data + bytes);
-	bytes += qq_get32(&external_group_id, data + bytes);
-	g_return_if_fail(internal_group_id > 0 && external_group_id);
+	bytes += qq_get32(&id, data + bytes);
+	bytes += qq_get32(&ext_id, data + bytes);
+	g_return_if_fail(id > 0 && ext_id);
 
-	group = qq_group_create_internal_record(gc, internal_group_id, external_group_id, NULL);
-	group->my_status = QQ_GROUP_MEMBER_STATUS_IS_ADMIN;
+	group = qq_group_create_internal_record(gc, id, ext_id, NULL);
+	group->my_role = QQ_ROOM_ROLE_ADMIN;
 	group->creator_uid = qd->uid;
 	qq_group_refresh(gc, group);
 
-	qq_group_activate_group(gc, internal_group_id);
-	qq_send_cmd_group_get_group_info(gc, group);
+	qq_send_room_cmd_only(gc, QQ_ROOM_CMD_ACTIVATE, id);
+	qq_update_room(gc, 0, group->id);
 
-	purple_debug(PURPLE_DEBUG_INFO, "QQ", "Succeed in create Qun, external ID %d\n", group->external_group_id);
+	purple_debug_info("QQ", "Succeed in create Qun, external ID %d\n", group->ext_id);
 
 	g = g_new0(gc_and_uid, 1);
 	g->gc = gc;
-	g->uid = internal_group_id;
+	g->uid = id;
 
 	purple_request_action(gc, _("QQ Qun Operation"),
 			    _("You have successfully created a Qun"),
 			    _
-			    ("Would you like to set up the Qun details now?"),
+			    ("Would you like to set up the detail information now?"),
 			    1,
 				purple_connection_get_account(gc), NULL, NULL,
 				g, 2,
@@ -387,54 +377,37 @@ void qq_group_process_create_group_reply(guint8 *data, gint len, PurpleConnectio
 			    _("Cancel"), G_CALLBACK(qq_do_nothing_with_gc_and_uid));
 }
 
-/* we have to activate group after creation, otherwise the group can not be searched */
-void qq_group_activate_group(PurpleConnection *gc, guint32 internal_group_id)
-{
-	guint8 data[16] = {0};
-	gint bytes = 0;
-	g_return_if_fail(internal_group_id > 0);
-
-	bytes = 0;
-	/* we create the simplest group, only group name is given */
-	/* 000 */
-	bytes += qq_put8(data + bytes, QQ_GROUP_CMD_ACTIVATE_GROUP);
-	/* 001-005 */
-	bytes += qq_put32(data + bytes, internal_group_id);
-
-	qq_send_group_cmd(gc, NULL, data, bytes);
-}
-
 void qq_group_process_activate_group_reply(guint8 *data, gint len, PurpleConnection *gc)
 {
 	gint bytes;
-	guint32 internal_group_id;
+	guint32 id;
 	qq_group *group;
 	g_return_if_fail(data != NULL);
 
 	bytes = 0;
-	bytes += qq_get32(&internal_group_id, data + bytes);
-	g_return_if_fail(internal_group_id > 0);
+	bytes += qq_get32(&id, data + bytes);
+	g_return_if_fail(id > 0);
 
 	/* we should have its info locally */
-	group = qq_group_find_by_id(gc, internal_group_id, QQ_INTERNAL_ID);
+	group = qq_room_search_id(gc, id);
 	g_return_if_fail(group != NULL);
 
-	purple_debug(PURPLE_DEBUG_INFO, "QQ", "Succeed in activate Qun %d\n", group->external_group_id);
+	purple_debug_info("QQ", "Succeed in activate Qun %d\n", group->ext_id);
 }
 
 void qq_group_manage_group(PurpleConnection *gc, GHashTable *data)
 {
-	gchar *internal_group_id_ptr;
-	guint32 internal_group_id;
+	gchar *id_ptr;
+	guint32 id;
 	qq_group *group;
 
 	g_return_if_fail(data != NULL);
 
-	internal_group_id_ptr = g_hash_table_lookup(data, "internal_group_id");
-	internal_group_id = strtol(internal_group_id_ptr, NULL, 10);
-	g_return_if_fail(internal_group_id > 0);
+	id_ptr = g_hash_table_lookup(data, QQ_ROOM_KEY_INTERNAL_ID);
+	id = strtol(id_ptr, NULL, 10);
+	g_return_if_fail(id > 0);
 
-	group = qq_group_find_by_id(gc, internal_group_id, QQ_INTERNAL_ID);
+	group = qq_room_search_id(gc, id);
 	g_return_if_fail(group != NULL);
 
 	/* XXX insert UI code here */
